# Blue Coconut version 1.0, Copyright 2004 Paul Mison.

use strict;
use warnings;

package Blue_Coconut::DAAP;

use Blue_Coconut::Guts;

use File::Spec;
use File::Path;

use Net::DAAP::Client;

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;

  my $self = {};
  bless ($self, $class);

  return $self;
}

sub connect {
  my $self = shift;
  my $info = shift;

  # reuse cache if we can
  unless ($self->{name} && $self->{name} eq $info->{name}) {

    my $daap = Net::DAAP::Client->new(SERVER_HOST => $info->{address},
                                      SERVER_PORT => $info->{port});
    push @{ $daap->{SONG_ATTRIBUTES} },
      qw( daap.songtracknumber daap.songtrackcount daap.songcompilation 
          daap.songdiscnumber daap.songdisccount );
  
    $daap->connect() 
      or Blue_Coconut::Guts->die_osa("No connection to '".$info->{name}."'",
                                     $daap->error(),
                                     "Cannot connect to shared library");

    $self->{daap} = $daap;
    $self->{name} = $info->{name};
  }
  return $self;
}

sub get_songs {
  my $self   = shift;
  my $daap   = $self->{daap};

  my $tracks = shift;
  my $songs = [];

  my $fields = $self->_lookup;

  DAAP: foreach my $song (values %{$daap->songs}) {
    ITUNES: foreach my $track (@{ $tracks }) {
      if ($track->{'size'} eq $song->{$fields->{'size'}}) {
        next ITUNES if $track->{matched};
        my $match = 1;

        # compare track size, album, artist and track number, if available
        foreach my $field (sort keys %{ $fields }) {
          $match = 0 if (defined $track->{ $field } &&
                         $track->{ $field } ne $song->{ $fields->{$field} });
        }

        if ($match) {
          push @{ $songs }, $song;
          $track->{matched} = 1;
          next DAAP;
        }
      }
    }
  }

  return $songs;
}

sub save_song {
  my $self   = shift;
  my $daap   = $self->{daap};

  my $song = shift;

  my $guts     = Blue_Coconut::Guts->new;
  my $save_dir = $guts->get_save_dir();

  # make song name
  my ($dir, $file, $ext) = $self->get_filename($song);

  # make folder to put file in, if necessary
  eval {  mkpath(File::Spec->catfile($save_dir, $dir)) };
  if ($@) {
    $guts->die_osa("File not downloaded",
                   "Cannot write to Music folder",
                   "Error: file not downloaded");
  }

  my $location = File::Spec->catfile($save_dir, $dir, $file);
  my $number = 1;
  while (-e join('.', $location, $ext)) {
    warn "File exists, adding number to filename";
    $location = File::Spec->catfile($save_dir, $dir, $file) . ".$number";
    $number++;
  }
  $location .= ".$ext";

  # TODO make this stuff display in BC interface
  my $data = $daap->save(undef, $song->{'dmap.itemid'});
  if ($daap->error) {
    $guts->die_osa("File not downloaded",
                   $daap->error,
                   "Error: $daap->error");
  }

  open (DOWNLOAD, ">$location") 
    or $guts->die_osa("File not saved",
                      "Cannot write to Music folder",
                      "Error: file not saved");
  binmode DOWNLOAD; # this is binary after all 
  print DOWNLOAD $data;
  close DOWNLOAD;

  return $location;
}  

sub get_filename {
  my $self = shift;
  my $song = shift;

  my ($file, $folder);
  
  my $artist = $song->{'daap.songcompilation'}
               ? 'Compliations'
               : $song->{'daap.songartist'};
  $artist = "Unknown Artist" unless $artist;

  my $album  = $song->{'daap.songalbum'}  || 'Unknown Album';
  $artist = "Compilations" if ($song->{'daap.songcompilation'});
                                   
  # TODO multiple discs
  if ($song->{'daap.songdisccount'} > 1) {
    $file = sprintf("%d-%02d %s", # that's more like iTunes
                    $song->{'daap.songdiscnumber'},
                    $song->{'daap.songtracknumber'},
                    $song->{'dmap.itemname'});
  } elsif ($song->{'daap.songtracknumber'}) {
    $file = sprintf("%02d %s", # that's more like iTunes
                    $song->{'daap.songtracknumber'},
                    $song->{'dmap.itemname'});
  } else {
    $file = $song->{'dmap.itemname'};
  }

  s!/!:!g for ($artist, $album, $file);

  $folder = File::Spec->catfile($artist, $album);

  return ($folder, $file, $song->{'daap.songformat'});
}

sub _lookup {
  return { 'size'   => 'daap.songsize',
           'title'  => 'dmap.itemname',
           'album'  => 'daap.songalbum',
           'artist' => 'daap.songartist',
           'track'  => 'daap.songtracknumber',
         }


}

1;
